//// sample of a telemarketing dialer.....

#include "PDDIAL.CH"
#include "INKEY.CH"

#define HWIRQ         5  // hardware interrupt of Dialogic board
#define MAXCHAN       4  // maximum number of channels
#define NUMRINGS      0  // TURN RING DETECT OFF

#define INITIATE      1  // array id for initiate code block
#define PROCESS       2  // array id for process code block

// File wide statics
STATIC lStateLoop,;      // terminates execution of the state machine
       nChan,;           // channel of current event
       nEvent,;          // current event
       M_TRIES,;
       M_LINES,;
       NEVENTDATA,;
       nMaxChan  := 0,;                             // maximum number of channels
       aCurBlock:={},;       // array of current initiate and process code blocks
       aFileHandle:={},;       // array of current file handles opened
       arecno:={}

//
FUNCTION StartUp(cHWIRQ,TRIES,DIALCHANS)
// initialize the state machine processing.
// can pass the hardware interrupt number from the DOS command line

LOCAL NDevType,;                                 // device type on which the error occured
      nErr,;                                     // init error code
      nHWIRQ,;                                   // hardware interrupt for board
      nInKey,;                                   // key input value
      X
// If no hardware interrupt was passed as a parameter, set the hardware
// interrupt value to the one defined by the manifest constant.
IF EMPTY(cHWIRQ)
   nHWIRQ := HWIRQ
ELSE
   // convert the hardware interrupt to a numeric.
   nHWIRQ := VAL(cHWIRQ)
ENDIF

IF EMPTY(TRIES)
   M_TRIES := 1
ELSE
   M_TRIES := VAL(TRIES)
ENDIF

IF EMPTY(DIALCHANS)
   M_Lines := 4
ELSE
   M_lINES := VAL(DIALCHANS)
ENDIF


CLEAR SCREEN

// Start the voice system
IF (nErr := pd_init(nHWIRQ, @nMaxChan,;
                    C_LC + C_OFFH + C_ONH, NUMRINGS)) == 0
  // there was no error doing the initialization of the driver, etc.

  FOR X := 1 TO nMaxChan
      AADD(aCurBlock,  {{ || OnHookInit() }, { || OnHookProc() }})
      AADD(aFileHandle, 0)
      AADD(ARECNO,0)
  NEXT

  // set globals to start with
  pd_global('loopsig', 1)                        // generate event on dropped line
  
  
  Use DIALDB
  INDEX ON DIALDB->STATUS+DIALDB->TRIES TO STATUS


  DO WHILE (nInKey := INKEY()) <> K_ESC
    

    // Did an event occur?
    IF pd_getEvent(@nChan, @nEvent, @nDevType, @nEventData) == -1
      
      WinMsg('Processing Events, ESC to exit')
      
      EVAL(aCurBlock[nChan, PROCESS])

      IF (nErr := EVAL(aCurBlock[nChan, INITIATE])) <> 0
         // process the error message
         pd_error(nErr)
      ENDIF
    ENDIF
  ENDDO

  /// IF WE SHUT DOWN THE SYSTEM, WE WILL RETAG THOSE RECORDS THAT ARE CURRENTLY
  //  IN MOTION SO THEY CAN BE RECALLED. WE WILL CHANGE THE CODE '6' TO 
  /// A ' ' 

  SELE DIALDB
  FOR X:=1 TO M_LINES
    IF ARECNO[X] > 0
       GOTO ARECNO[X]
       REPLACE DIALDB->STATUS WITH ' '  
       REPLACE DIALDB->TRIES WITH ' '
    ENDIF
  NEXT

  // Shut down the voice system
  IF pd_deinit(nMaxChan) <> 0
    WinMsg( ALLTRIM(pd_d4xerr(nErr)) )
  ENDIF
ELSE
  WinMsg(  ALLTRIM(pd_d4xerr(nErr)) )
  wait
ENDIF


RETURN NIL









//
FUNCTION OnHookInit()
// Initiate function to set the channel on hook.
LOCAL nRetVal
pd_dispMsg('OnHookInit')
ARECNO[NCHAN]:=0 // RESET RECNO VAR
// Set the current channel on hook
nRetVal := pd_setHook(nChan, H_ONH)
RETURN nRetVal

//
FUNCTION OnHookProc()
// Process function to set the channel on hook.
pd_dispMsg('OnHookProc')
IF NCHAN <=M_LINES
   aCurBlock[nChan] := {{ || PAUSEInit() }, { || PAUSEProc() }}
ELSE
  aCurBlock[nChan] := {{ || LIMBOInit() }, { || LIMBOProc() }}
ENDIF
RETURN NIL



//
FUNCTION PAUSEInit()
LOCAL NRETVAL
pd_dispMsg('PAUSE TWO SECONDS') 
PD_RESET()
PD_GLOBAL('MAXSEC',2)
NRETVAL:=PD_GETDTMFS(NCHAN)
RETURN NRETVAL

//
FUNCTION PAUSEProc()
IF NEVENT=T_TIME
    aCurBlock[nChan] := {{ || DIALInit() }, { || DIALProc() }}  
ELSE
    aCurBlock[nChan] := {{ || ONHOOKInit() }, { || ONHOOKProc() }}  
ENDIF
RETURN NIL








//
FUNCTION DIALInit()
local Nretval
pd_dispMsg('Looking for Number to dial')
Sele Dialdb
set soft on
seek ' '
if val(dialdb->status) < 4 .AND. VAL(DIALDB->TRIES) < M_TRIES .AND. .NOT. EOF()
   REPLACE DIALDB->TRIES WITH LTRIM(STR(VAL(DIALDB->TRIES)+1))
   REPLACE DIALDB->STATUS WITH '6' ///  RECORD CURRENTLY BEING DIALED, DONT TOUCH
   ARECNO[NCHAN]=RECNO()
   NRETVAL:=PD_CALLP(NCHAN,DIALDB->PHONE_NUM)
ELSE
   NRETVAL:=PD_PUTEVENT(NCHAN,500,0)
ENDIF
RETURN NRETVAL

//
FUNCTION DIALProc()
pd_dispMsg('DialProc')
SELE DIALDB
GOTO ARECNO[NCHAN] 
IF nEvent == T_CATERM // CALL PROGRESS TERMINATED 
          
     DO CASE
       CASE NEVENTDATA=CA_CONN
         REPLACE DIALDB->STATUS WITH '7' /// MESSAGE PLAYED
         aCurBlock[nChan] := {{ || playmessInit() }, { || PlaymessProc() }} 
       CASE NEVENTDATA=CA_BUSY
         REPLACE DIALDB->STATUS WITH '1'  // BUSY
          aCurBlock[nChan] := {{ || ONHOOKInit() }, { || ONHOOKProc() }}
       CASE NEVENTDATA=CA_NOAN 
         REPLACE DIALDB->STATUS WITH '2'  // NO ANSWER
          aCurBlock[nChan] := {{ || ONHOOKInit() }, { || ONHOOKProc() }}
       CASE NEVENTDATA=CA_NORNG 
         REPLACE DIALDB->STATUS WITH '3' // NO RING
          aCurBlock[nChan] := {{ || ONHOOKInit() }, { || ONHOOKProc() }}
       CASE NEVENTDATA=CA_OPINT 
         REPLACE DIALDB->STATUS WITH '4'  // OPERATOR INTERCEPT
          aCurBlock[nChan] := {{ || ONHOOKInit() }, { || ONHOOKProc() }}
       OTHERWISE
         REPLACE DIALDB->STATUS WITH '5' // OTHER ERROR  
      ENDCASE

ELSEIF NEVENT=500
  //COULD NOT FIND ANY MORE, GO TO LIMBO
  aCurBlock[nChan] := {{ || LIMBOInit() }, { || LIMBOProc() }}
ELSE
  REPLACE DIALDB->STATUS WITH '0'   
  aCurBlock[nChan] := {{ || ONHOOKInit() }, { || ONHOOKProc() }}
ENDIF

RETURN NIL






//
FUNCTION LIMBOInit()
pd_dispMsg('CHANNEL IN LIMBO')   
RETURN 0
//
FUNCTION LIMBOProc()
RETURN NIL






//
FUNCTION PlayMESSInit(cFile)
LOCAL nHandle,nRetVal
pd_dispMsg('Play A MESSAGE' )
PD_RESET()
// Play the file
nRetVal := pd_play(nChan, 'TEST.VOX', @nHandle)
aFileHandle[nChan] := nHandle
RETURN nRetVal

//
FUNCTION PlayMESSProc()
// Close the file we played
pd_VOXClose(aFileHandle[nChan])
aCurBlock[nChan] := {{ || OnHookInit() }, { || OnHookProc() }}
RETURN NIL







//
FUNCTION WinMsg(cMessage)
// display informative excuse for processing pauses
// this is really a poor excuse for a message window, but its simple!

LOCAL nLeft,;
      nRight

CLEAR SCREEN

nLeft  := MAXCOL()/2 - LEN(cMessage)/2 - 1
nRight := MAXCOL()/2 + LEN(cMessage)/2 + 2

// draw a box
@ 9, nLeft TO 11, nRight + 1 DOUBLE COLOR '+R/B'

@ 10, nLeft + 1 SAY PADC(cMessage, nRight - nLeft) COLOR 'R/B'

RETURN NIL

