/**
  Ŀ
 ݳ                                                                          
 ݳ   Program : HostFLoc.cls                                                 
 ݳ   Purpose : Fueling Location class object defintions and methods         
 ݳ Written By: Peter Steele                                                 
 ݳThe procedures in this file are Copyright (C) 1988-1992 by the            
 ݳPinnacle dEVELOPMENT Corporation. These programs are the sole property of 
 ݳPinnacle dEVELOPMENT Corporation. They contain confidential information   
 ݳand are a trade secret of Pinnacle dEVELOPMENT Corporation. They are being
 ݳsupplied to you with the express understanding that the information       
 ݳcontained herein be held in confidence by you.  This software is not to be
 ݳcopied, distributed, or displayed to third parties without the express    
 ݳwritten consent of Pinnacle dEVELOPMENT Corporation, and must be returned 
 ݳupon request.  Any use of these programs without the express written      
 ݳconsent of Pinnacle dEVELOPEMENT Corporation, is strictly forbidden.      
 ݳThis software and any companion  documentation are protected by U.S.      
 ݳCopyright law (Title 17 United States Code).  Unauthorized reproduction   
 ݳand/or sale may result in imprisonment of up to one year and fines of up  
 ݳto $10,000 per infraction (17 USC 506).  Copyright infringers may also be 
 ݳsubject to civil liability.  Pinnacle dEVELOPMENT is an independent       
 ݳcorporation incorporated under the laws of the state of Texas, U.S.A.     
 
 

   This file contains methods which override those methods in the OASIS
   store level flocation.cls.

REVISION HISTORY:

 Date        Revision
01-04-93 PS Changes for Class(y) v2.0.
                                                                            */

#include "PINNSTD.CH"
#include "CLASS(Y).CH"

//

CREATE CLASS HostFloc FROM FLocation
   EXPORT:

   VAR cStore                          // store number this flocation belongs to
   VAR cTerritory                      // group/territory this store belongs to
   VAR dCurrent                        // current date to post to

   VAR lSticksLoaded                   // are stick readings loaded?
   VAR lDelivLoaded                    // are deliveries loaded?
   VAR lMetersLoaded                   // are meters loaded?

   METHOD Init                         // initialze function

   METHOD LoadTanks                    // load tank objects
   METHOD LoadGrades                   // load grade objects

   METHOD LoadSticks                   // load stick reading data
   METHOD SaveSticks                   // save stick reading data

   METHOD InitMeters                   // initialize meter objects
   METHOD LoadMeters                   // load meter objects with data
   METHOD SaveMeters                   // save meter data to TPUMP

   METHOD LoadHistory                  // load history data

   METHOD LoadDeliveries               // load delivery data
   METHOD SaveDeliveries               // save delivery data
   METHOD UpdateDeliveries             // update delivery total for the tank
   METHOD GetDelivery                  // get a particular delivery

   METHOD GetEnding                    // ending inventory for day/tank

   HIDDEN:

END CLASS

//
METHOD FUNCTION Init(),;
                    ()

::lMetersLoaded := ::lSticksLoaded := ::lDelivLoaded := .F.

RETURN self

//
METHOD FUNCTION LoadTanks()
// Load tank information into an objects.

LOCAL oTank


SETAREA('HTANKS', 'HTANKS')

// seek the first record for the store
HTANKS->( DBSEEK( ::cTerritory + ::cStore) )

// Add to tank info arrays
DO WHILE HTANKS->elsgroup == ::cTerritory .AND. ;
         HTANKS->elsstore == ::cStore .AND. ;
         .NOT. HTANKS->(EOF())

  // we will only load active tanks
  IF HTANKS->active
     oTank := HTank():New(HTANKS->tank_num,  HTANKS->product,  HTANKS->man_tank,;
                          HTANKS->chart_num, HTANKS->comp_num, HTANKS->capacity)
     oTank:nNameIdx := HTANKS->name_idx

     ::super:AddTank(oTank)
  ENDIF
  HTANKS->(DBSKIP())
ENDDO  

CLOSE HTANKS

RETURN self

//
METHOD FUNCTION LoadGrades()
// load grade objects for a store
LOCAL oGrade

// Load grade information into array.
SETAREA('HGRADES', 'HGRADES')

// locate to the first record for the store
HGRADES->(DBSEEK( ::cTerritory + ::cStore ))

DO WHILE HGRADES->elsstore == ::cStore .AND. ;
         HGRADES->elsgroup == ::cTerritory .AND. ;
         .NOT. HGRADES->(EOF())
  // create a new grade object
  oGrade := Hgrade():New()

  // assign its instance variables.
  oGrade:nGrade       := HGRADES->grade_num       // grade number (record number)
  oGrade:cName        := HGRADES->grade           // grade name
  oGrade:nPrice       := HGRADES->price           // price per/gallon/litre
  oGrade:nTank1       := HGRADES->tank1           // first (or only) blend tank
  oGrade:nTank2       := HGRADES->tank2           // second (blend) tank
  oGrade:nBlendRatio  := HGRADES->blendratio /100 // blend expressed as a factor
  oGrade:nCompNumber  := HGRADES->comp_num        // system compare number for PPG
  oGrade:nNameIdx     := HGRADES->name_idx        // name file index
  HGRADES->(DBSKIP())
  ::super:AddGrade(oGrade)
ENDDO

CLOSE HGRADES

RETURN self

//
METHOD FUNCTION LoadSticks()
// Load stick information into array of objects.

LOCAL nTank,;
      oStick,;                                   // stick reading object
      oTank                                      // tank object

SETAREA('TSTICK', 'TSTICK')

// add a new stick object for each tank
FOR nTank := 1 TO ::nTankCount
   // all values initially zero
   // add the stick object to its tank object
   ::super:GetTank(nTank):SetStick( Stick():New(nTank) )
NEXT

// seek the first record for the store/date
TSTICK->(DBSEEK( ::cTerritory + ::cStore + DTOS(::dCurrent) ))

DO WHILE TSTICK->elsdate == ::dCurrent .AND. ;
         TSTICK->elsstore == ::cStore .AND. ;
         TSTICK->elsgroup == ::cTerritory .AND. ;
         .NOT. TSTICK->(EOF())
  // add any stick reading values to the stick object
  oTank  := ::super:GetTank(TSTICK->tank_num)
  IF oTank <> NIL
     oStick := oTank:GetStick()               // get a pointer to the stick object
     oStick:nGasDepth    := TSTICK->gas_inch
     oStick:nWaterDepth  := TSTICK->water_inch
     oStick:nGasVolume   := TSTICK->gas_amt
     oStick:nWaterVolume := TSTICK->water_amt
     oStick:nStickCount++
  ENDIF
  TSTICK->(DBSKIP())
ENDDO

CLOSE TSTICK

::lSticksLoaded := .T.

RETURN self

//
METHOD FUNCTION SaveSticks()
LOCAL nTank,;
      oStick                                     // stick reading object

// Load stick information into database.
SETAREA('TSTICK', 'TSTICK')

FOR nTank := 1 TO ::nTankCount
   oStick := ::super:GetTank(nTank):GetStick()
   TSTICK->(DBSEEK(::cTerritory + ::cStore + DTOS(::dCurrent) + STR(nTank,2), .F.))
   IF TSTICK->(FOUND())
      IF RecLock(STDWAIT)
         REPLACE TSTICK->gas_inch   WITH oStick:nGasDepth,;
                 TSTICK->water_inch WITH oStick:nWaterDepth,;
                 TSTICK->gas_amt    WITH oStick:nGasVolume,;
                 TSTICK->water_amt  WITH oStick:nWaterVolume,;
                 TSTICK->elstime    WITH TIME()
         TSTICK->(DBUNLOCK())
      ENDIF
   ELSE
      IF TSTICK->(AddRec(STDWAIT))
         REPLACE TSTICK->day_num    WITH 1,;
                 TSTICK->elsgroup   WITH ::cTerritory,;
                 TSTICK->elsstore   WITH ::cStore,;
                 TSTICK->elsdate    WITH ::dCurrent,;
                 TSTICK->elstime    WITH TIME(),;
                 TSTICK->console_id WITH 9,;
                 TSTICK->gas_inch   WITH oStick:nGasDepth,;
                 TSTICK->water_inch WITH oStick:nWaterDepth,;
                 TSTICK->gas_amt    WITH oStick:nGasVolume,;
                 TSTICK->water_amt  WITH oStick:nWaterVolume,;
                 TSTICK->tank_num   WITH nTank
         TSTICK->(DBUNLOCK())
      ENDIF
   ENDIF
NEXT

CLOSE TSTICK

RETURN self

//x
METHOD FUNCTION InitMeters
// initialize meter objects
LOCAL oMeter

SETAREA('HMETERS', 'HMETERS')

HMETERS->(DBSEEK(::cTerritory + ::cStore))

DO WHILE HMETERS->elsgroup == ::cTerritory .AND. ;
         HMETERS->elsstore == ::cStore     .AND. ;
          .NOT. HMETERS->(EOF())
   oMeter := Meter():New()
   oMeter:nFPosition  := HMETERS->fposition
   oMeter:nMeter      := HMETERS->meter
   oMeter:cName       := HMETERS->desc
   oMeter:nGrade      := HMETERS->grade
   oMeter:nGalCompare := HMETERS->gal_compar
   oMeter:nDolCompare := HMETERS->dol_compar
   oMeter:lResetting  := HMETERS->resetting
   ::AddMeter(oMeter)
   // default the calculated price to come up as the posted price
   oMeter:nCalcPrice  := ::super:GetMeterGrade(oMeter:nGrade):nPrice
   HMETERS->(DBSKIP())
ENDDO

CLOSE HMETERS

RETURN self

//
METHOD FUNCTION LoadMeters
// load meter readings from TPUMP.DBF
LOCAL bMatch,;                                   // match block
      nMeter,;                                   // meter loop counter
      oMeter

SETAREA('TPUMP', 'TPUMP')

// get the previous readings
FOR nMeter := 1 TO ::nMeterCount
    oMeter := ::super:GetMeter(nMeter)
    TPUMP->(DBSEEK(::cTerritory + ::cStore + DTOS(::dCurrent) + ;
                   STR(oMeter:nFPosition, 2) + STR(oMeter:nMeter, 2) + ;
                   ' 0'))
    IF TPUMP->(FOUND())
       oMeter:nEndGallons := TPUMP->gallons
       oMeter:nEndDollars := TPUMP->gassales
       oMeter:nEndPrice   := TPUMP->new_price
       oMeter:nTstGallons := TPUMP->gals_test
       oMeter:nTstDollars := TPUMP->dols_test

       // initialize things for the pretty display
       oMeter:CalcSales()
       oMeter:CalcGallons()
       oMeter:calcPrice()
   ENDIF
NEXT

::super:UpdateMeterTotals()

CLOSE TPUMP

::lMetersLoaded := .T.

RETURN self

//
METHOD FUNCTION SaveMeters()
// save meters to the database
LOCAL nMeter,;
      oGrade,;
      oMeter

SETAREA('HGRADES', 'HGRADES')
SETAREA('TPUMP', 'TPUMP')

FOR nMeter := 1 TO ::nMeterCount
   oMeter := ::super:GetMeter(nMeter)
   oGrade := ::super:getGrade(oMeter:nGrade)

   TPUMP->(DBSEEK(::cTerritory + ::cStore + DTOS(::dCurrent) + ;
                  STR(oMeter:nFposition, 2) + STR(oMeter:nMeter, 2) + ' 0' ))
   IF .NOT. TPUMP->(FOUND())
      TPUMP->(DBAPPEND())
       REPLACE TPUMP->day_num    WITH 1,;
               TPUMP->elsgroup   WITH ::cTerritory,;
               TPUMP->elsstore   WITH ::cStore,;
               TPUMP->elsdate    WITH ::dCurrent,;
               TPUMP->elstime    WITH TIME(),;
               TPUMP->console_id WITH 9,;
               TPUMP->fposition  WITH oMeter:nFPosition,;
               TPUMP->hose       WITH oMeter:nMeter
   ENDIF
   REPLACE TPUMP->grade     WITH oMeter:nGrade,;
           TPUMP->sequence  WITH 0,;
           TPUMP->tank1     WITH oGrade:nTank1,;
           TPUMP->tank2     WITH oGrade:nTank2,;
           TPUMP->old_price WITH oMeter:nBegPrice,;
           TPUMP->new_price WITH oMeter:nEndPrice,;
           TPUMP->gallons   WITH oMeter:nEndGallons,;
           TPUMP->gassales  WITH oMeter:nEndDollars,;
           TPUMP->gals_gone WITH oMeter:nSlsGallons,;
           TPUMP->dols_gone WITH oMeter:nSlsDollars,;
           TPUMP->gals_test WITH oMeter:nTstGallons,;
           TPUMP->dols_test WITH oMeter:nTstDollars

    // seek soft
    HGRADES->(DBSEEK( ::cTerritory + ::cStore + DTOS(::dCurrent), .T.))
    REPLACE HGRADES->store_pric WITH oGrade:oSales:nEnteredPrice

NEXT

CLOSE TPUMP
CLOSE HGRADES

RETURN self

//
METHOD FUNCTION LoadHistory()
// load gas history data into the objects (sorted by tank)

LOCAL cKey,;
      nFirstTank,;
      oHistory

nFirstTank  := ::super:getTank(1):nTank

SETAREA('HGASRECO', 'HGASRECO')

// locate to the oldest record we will be using (softseek)
cKey := ::cTerritory + ::cStore + STR(nFirstTank, 2) + IIF(::nNumDays == 0, DTOS(BOM(::dCurrent)), DTOS(::dCurrent - ::nNumDays ) )

HGASRECO->( DBSEEK(cKey, .T.) )

DO WHILE HGASRECO->elsgroup == ::cTerritory .AND. ;
         HGASRECO->elsstore == ::cStore    .AND. ;
         HGASRECO->elsdate <   ::dCurrent .AND. ;
         .NOT. HGASRECO->(EOF())
   oHistory := GasHistory():New()
   oHistory:dDate          := HGASRECO->elsdate
   oHistory:nTank          := HGASRECO->tank
   oHistory:nOS            := HGASRECO->overshort
   oHistory:nDeliveryTotal := HGASRECO->deliveries
   oHistory:nMeterTotal    := HGASRECO->metertotal
   oHistory:nEndingInv     := HGASRECO->actualinv

   AADD(::aHistory, oHistory)
   HGASRECO->(DBSKIP())
ENDDO

CLOSE HGASRECO

RETURN self

//
METHOD FUNCTION GetEnding(nTank, dDate)
LOCAL nBeginInv         := 0,;                   // beginning inventory for the tank on the day
      nFound                                     // array element where the day/tank was found

nFound := ASCAN(::aHistory, {|oHistory| oHistory:dDate == dDate .AND.;
                                        oHistory:nTank == nTank})
IF nFound <> 0
   nBeginInv := ::aHistory[nFound]:nEndingInv
ENDIF

RETURN nBeginInv

//
METHOD FUNCTION LoadDeliveries()
// load delivery objects from TGASDELV
LOCAL oDelivery,;
      oTank

SETAREA('TGASDELV', 'TGASDELV')

TGASDELV->(DBSEEK(::cTerritory + ::cStore) )

DO WHILE TGASDELV->elsdate == ::dCurrent .AND. ;
         TGASDELV->elsstore == ::cStore .AND. ;
         TGASDELV->elsgroup == ::cTerritory .AND. ;
         .NOT. TGASDELV->(EOF())
   oDelivery       := DelObj():New(self)
   oDelivery:dDate       := TGASDELV->elsdate
   oDelivery:nTank       := TGASDELV->tank_num
   oDelivery:nDelivery   := TGASDELV->delivery
   oDelivery:nSequence   := TGASDELV->sequence
   oDelivery:cInvoice    := TGASDELV->invoice
   oDelivery:nVendor     := TGASDELV->vendor_num
   oDelivery:cCarrier    := TGASDELV->carrier
   oDelivery:cTerminal   := TGASDELV->terminal
   oDelivery:cBOL        := TGASDELV->bol_num
   oDelivery:nCost       := TGASDELV->inv_cost
   oDelivery:nRetail     := TGASDELV->inv_retail
   oDelivery:nNetGallons := TGASDELV->net_gallon
   oDelivery:nGrsGallons := TGASDELV->grs_gallon
   oDelivery:nUnitCost   := TGASDELV->unit_cost
   oDelivery:nUnitRetail := TGASDELV->unitretail
   oDelivery:nDegrees    := TGASDELV->degrees

   // add a pointer for the tank object
   oTank := ::getTank(oDelivery:nTank)

   // add a pointer to the delivery to the fueling location
   // (this adds a pointer to the tank too!)
   ::super:addDelivery(oDelivery)

   // set the last delivery instance variable
   ::nLastDelivery := MAX(::nLastDelivery, oDelivery:nDelivery)

   TGASDELV->(DBSKIP())
ENDDO

CLOSE TGASDELV

::lDelivLoaded := .T.

RETURN self

//
METHOD FUNCTION SaveDeliveries()
LOCAL lPack             := .F.,;                 // flag to pack the file
      nDelivery,;                                // delivery number loop counter
      nTank,;                                    // tank number loop counter
      oDelivery

SETAREA('TGASDELV', 'TGASDELV')

FOR nDelivery := 1 TO LEN(::aDeliveries)
   oDelivery  := ::aDeliveries[nDelivery]
   // seek the record in TGASDELV
   TGASDELV->(DBSEEK(::cTerritory + ::cStore + DTOS(oDelivery:dDate) + STR(oDelivery:nDelivery, 2) ))
   IF .NOT. TGASDELV->(FOUND()) .AND. .NOT. oDelivery:lDeleted
      TGASDELV->(DBAPPEND())
      REPLACE TGASDELV->day_num    WITH 1,;
              TGASDELV->elsgroup   WITH ::cTerritory,;
              TGASDELV->elsstore   WITH ::cStore,;
              TGASDELV->elsdate    WITH ::dCurrent,;
              TGASDELV->elstime    WITH TIME(),;
              TGASDELV->console_id WITH 9
   ENDIF
   IF oDelivery:lDeleted
      lPack := .T.
      TGASDELV->(DBDELETE())
   ELSE
      // do replaces
      TGASDELV->elsdate    := oDelivery:dDate
      TGASDELV->tank_num   := oDelivery:nTank
      TGASDELV->delivery   := oDelivery:nDelivery
      TGASDELV->sequence   := oDelivery:nSequence
      TGASDELV->invoice    := oDelivery:cInvoice
      TGASDELV->vendor_num := oDelivery:nVendor
      TGASDELV->carrier    := oDelivery:cCarrier
      TGASDELV->terminal   := oDelivery:cTerminal
      TGASDELV->bol_num    := oDelivery:cBOL
      TGASDELV->inv_cost   := oDelivery:nCost
      TGASDELV->inv_retail := oDelivery:nRetail
      TGASDELV->net_gallon := oDelivery:nNetGallons
      TGASDELV->grs_gallon := oDelivery:nGrsGallons
      TGASDELV->unit_cost  := oDelivery:nUnitCost
      TGASDELV->unitretail := oDelivery:nUnitRetail
      TGASDELV->degrees    := oDelivery:nDegrees
      TGASDELV->fuel_befor := oDelivery:FuelBefore()
      TGASDELV->fuel_after := oDelivery:FuelAfter()
      TGASDELV->watr_befor := oDelivery:WaterBefore()
      TGASDELV->watr_after := oDelivery:WaterAfter()
      TGASDELV->stik_osgrs := oDelivery:GrossOS()
      TGASDELV->stik_osnet := oDelivery:NetOS()
   ENDIF

NEXT delivery

IF lPack
   SELECT TGASDELV
   PACK
ENDIF

CLOSE TGASDELV

RETURN self

//
METHOD FUNCTION UpdateDeliveries(nTank)
// update delivery totals for the tank
LOCAL bTotalBlock,;                              // code block to add up deliveries
      nDelivery,;                                // delivery number loop counter
      nTotal            := 0,;                   // total gallons delivered
      oDelivery,;                                // temp delivery object
      oTank                                      // current tank object

// get the current tank object
oTank    := ::getTank(nTank)

// set the code block to total up deliveries based on the system parameter
// stored in the fueling location object.
IF ::lNetDeliveries
   // use net delivery total for reconciliation
   bTotalBlock := {|| nTotal += oDelivery:nNetGallons}
ELSE
   bTotalBlock := {|| nTotal += oDelivery:nGrsGallons}
ENDIF

FOR nDelivery := 1 TO oTank:nDelivCount
   // get a pointer to the delivery object
   oDelivery := oTank:getDelivery(nDelivery)
   IF .NOT. oDelivery:lDeleted .AND. oDelivery:dDate == ::dCurrent
      // if the delivery object has not been deleted and its for the current date
      EVAL(bTotalBlock)
   ENDIF
NEXT

RETURN nTotal

//
METHOD FUNCTION GetDelivery(nTank)
// returns the delivery object for a tank or NIL if none found
LOCAL nDelivery,;                                // loop counter
      oDelivery         := NIL                   // delivery object

FOR nDelivery := 1 TO LEN(::aDeliveries)
    IF ::aDeliveries[nDelivery]:nTank == nTank
       EXIT
       oDelivery := ::aDeliveries[nDelivery]
    ENDIF
NEXT

RETURN oDelivery

