// a very simple conversation recorder. Begins recording when phone line
// is connected. good for recording phone calls.
////////////////////////////////////////////////////////////////////////////
#include "PDDIAL.CH"
#include "INKEY.CH"

#define HWIRQ         5  // hardware interrupt of Bicom board

#define INITIATE      1  // array id for initiate code block
#define PROCESS       2  // array id for process code block

// File wide statics
STATIC lStateLoop,;      // terminates execution of the state machine
       nChan,;           // channel of current event
       nEvent,;          // current event
       aCurBlock,;       // array of current initiate and process code blocks
       aFileHandle       // array of current file handles opened

//
FUNCTION StartUp(cHWIRQ)
// initialize the state machine processing.
// can pass the hardware interrupt number from the DOS command line

LOCAL nChannel,;                                 // loop counter for initialization of the channels
      nDevType,;                                 // device type on which the error occured
      nEventData,;                               // data variable passed back with the event code
      nErr,;                                     // init error code
      nHWIRQ,;                                   // hardware interrupt for board
      nInKey,;                                   // key input value
      nMaxChan          := 0                     // maximum number of channels

// If no hardware interrupt was passed as a parameter, set the hardware
// interrupt value to the one defined by the manifest constant.
IF EMPTY(cHWIRQ)
   nHWIRQ := HWIRQ
ELSE
   // convert the hardware interrupt to a numeric.
   nHWIRQ := VAL(cHWIRQ)
ENDIF

CLEAR SCREEN

// Start the voice system
IF (nErr := pd_init(nHWIRQ, @nMaxChan,;
                    C_LCON + C_OFFH + C_ONH, 0)) == 0
  // there was no error doing the initialization of the driver, etc.

  // Set initial state: re-initialize channel array and file handle arrays.
  
  aCurBlock   := {}
  aFileHandle := {}

  FOR nChannel := 1 TO nMaxChan
      // One array element per channel.  Each array element contains
      // a two element array containing the next initiate function and
      // the next process function.  All channels start on-hook.
      AADD(aCurBlock,  {{ || OnhookInit() }, { || OnHookProc() }})

      // a storage space to keep the channel's file handle
      AADD(aFileHandle, 0)
  NEXT

  // set globals to start with
  pd_global('loopsig', 1)                        // generate event on dropped line
  pd_global('termdtmf', '@')                     // terminate on any character

  // Loop to call the voice handler
  // processing events with the state machine

  // yo, this is the "state machine" processing loop>>>>>
  DO WHILE (nInKey := INKEY()) <> K_ESC


    // Did an event occur?
    IF pd_getEvent(@nChan, @nEvent, @nDevType, @nEventData) == -1
      // Placing this message in here slows us down a little.  It's
      // certainly not necessary.  Its just an example of how you can
      // put your own function calls which can do other types of
      // processing inside the state machine loop!
      WinMsg('Processing Events, ESC to exit')

      // Process the event and save the next channel state
      // (execute the "PROCESS" function)
      EVAL(aCurBlock[nChan, PROCESS])

      // execute the INITIATE function.  (this sets the code blocks
      // defining the next activity to execute.)

      IF (nErr := EVAL(aCurBlock[nChan, INITIATE])) <> 0
         // process the error message
         pd_error(nErr)
      ENDIF
    ENDIF
  ENDDO

  // Shut down the voice system
  IF pd_deinit(nMaxChan) <> 0
    WinMsg( ALLTRIM(pd_d4xerr(nErr)) )
  ENDIF
ELSE
  WinMsg(  ALLTRIM(pd_d4xerr(nErr)) )
ENDIF


RETURN NIL


///// put channel on hook to start initialy 
//
FUNCTION OnHookInit()
// Initiate function to put  the channel on hook.
LOCAL nRetVal

pd_dispmsg('OnHookInit'+"--"+ltrim(str(nchan)))
nRetVal := pd_setHook(nChan, H_ONH)

RETURN nRetVal

//
FUNCTION OnHookProc()
pd_dispmsg('ONhookProc'+"--"+ltrim(str(nchan)))

// When channel goes off hook proceed
   aCurBlock[nChan] := {{ || OffhookInit() }, { || OffhookProc() }}

RETURN NIL




///// Now Take it off hook and leave it 
//
FUNCTION OffHookInit()
LOCAL nRetVal

pd_dispmsg('OffHookInit'+"--"+ltrim(str(nchan)))
nRetVal := pd_setHook(nChan, H_OFFH)

RETURN nRetVal

//
FUNCTION OffHookProc()
pd_dispmsg('OffHookProc'+"--"+ltrim(str(nchan)))

// When channel goes off hook proceed
aCurBlock[nChan] := {{ || loopInit() }, { || loopProc() }}
RETURN NIL






/// Wait for Loop Event
//
FUNCTION LoopInit()
// Initiate function to wait for Loop transision Event.
pd_dispmsg('LoopInit'+"--"+ltrim(str(nchan)))
// sit in loop and wait for the loop event
RETURN E_SUCC  // always successful

//
FUNCTION LoopProc()
pd_dispmsg('LoopProc'+"--"+ltrim(str(nchan)))

IF nEvent == T_LCON  
// loop detected, go record it now
  aCurBlock[nChan] := {{ || recordInit() }, { || recordProc() }}
ELSE
  //  Unexpected event detected, must wait for another loop event
  aCurBlock[nChan] := {{ || loopinit() }, { || loopProc() }}
ENDIF

RETURN NIL





//
FUNCTION RecordInit()
// Initiate function to handle Recording the  message.
LOCAL nHandle,;
      nRetVal
pd_dispmsg('Now recording Testfil.vox on chan'+ltrim("--"+ltrim(str(nchan))))

// turn extended read write block on
pd_global("isxrwb",1)

// set the record to term if silence should exceed 5 seconds..
pd_global("Maxsil",5)

// set the record to term is there is more than 6 sec of non silence
pd_global("Maxnsil",6)

// Message can be no more than 60 seconds long
pd_global("Maxsec",60)

nRetVal := pd_Recfile(nChan, 'Testfil'+ltrim(str(nchan))+'.VOX', @nHandle , 0 , .F. )
aFileHandle[nChan] := nHandle                    // save the handle
RETURN nRetVal

//
FUNCTION RecordProc()
// Process function after the menu message has played.

pd_dispmsg('Recording Terminated on chan'+ltrim("--"+ltrim(str(nchan))))
IF nEvent == T_LCTERM .OR. nEvent == T_TERMDT   // here check for other states
  // normal file play completion, go into menu input process.
  aCurBlock[nChan] := {{ || Loopinit() }, { || LoopProc() }}
ELSE
  // Something strange happend, Go back to waiting for loop event.
  aCurBlock[nChan] := {{ || LoopInit() }, { || LoopProc() }}
ENDIF

// Close the most recently opened file
pd_VOXClose(aFileHandle[nChan])

RETURN NIL





//
FUNCTION WinMsg(cMessage)
// display informative excuse for processing pauses
// this is really a poor excuse for a message window, but its simple!

LOCAL nLeft,;
      nRight

CLEAR SCREEN

nLeft  := MAXCOL()/2 - LEN(cMessage)/2 - 1
nRight := MAXCOL()/2 + LEN(cMessage)/2 + 2

// draw a box
@ 9, nLeft TO 11, nRight + 1 DOUBLE COLOR '+R/B'

@ 10, nLeft + 1 SAY PADC(cMessage, nRight - nLeft) COLOR 'R/B'

RETURN NIL

